==> seplib-6.3.5.tar.gz: seplib-6.3.5/seplib_base/lib/corelibs/superset/sect_transfer_mpi.c <==
#define SET_SDOC 1
#include<sitedef.h>
#include<superset.h>
#include<../include/superset_internal.h>
#ifdef SEP_MPI
#include<mpi.h>
#endif
#ifndef MAX
#define MAX(a,b) ( ((a)>(b)) ? (a):(b) )
#endif
#ifndef MIN
#define MIN(a,b) ( ((a)<(b)) ? (a):(b) )
#endif



#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_broadcast_headers(char *sep3dname, int ifrom)
_XFUNCPROTOEND
#endif
{
sep_3d *info;
#ifdef SEP_MPI
MPI_Status status;
MPI_Comm comm;
#endif
char usage[256],data[256],file[256];
int nsz,i,k,iam,impi;
int n,nbuf;
float o,d;
char temp1[256],temp2[256],temp3[256];
char *buf;


#ifdef SEP_MPI
  MPI_Comm_rank(MPI_COMM_WORLD, &impi);
if(ifrom==impi){
  info = tag_info_sep3d(sep3dname, INQUIRE);  /* get info on this tag */
  if(info == SEPNULL)
    return (sepwarn(INVALID_STRUC,"tag:%s 2invalid struc\n",sep3dname));
  
  nbuf=SEP3D_nconvert(info);

}
else{
  info = tag_info_sep3d(sep3dname, INQUIRE);  /* get info on this tag */
  if(info == SEPNULL){
    info = tag_info_sep3d(sep3dname, SCRATCH);  /* get info on this tag */
     if(info == SEPNULL)
        return (sepwarn(INVALID_STRUC,"tag:%s 1invalid struc\n",sep3dname));
   }
  

}
  MPI_Bcast( &nbuf, 1, MPI_INT,  ifrom,MPI_COMM_WORLD); 
  sep_mpi_stop();
  buf=(char*) malloc(sizeof(char)*(nbuf));
if(ifrom==impi){
    nbuf=0;
   if(0!=SEP3D_convert(info,&nbuf,buf))
     return(sepwarn(NOT_MET,"trouble converting tag  %s \n",info->name));
}
  MPI_Bcast((void*)buf,nbuf,MPI_BYTE, ifrom, MPI_COMM_WORLD);
sep_mpi_stop();
if(ifrom!=impi){
    nbuf=0;
   if(0!=SEP3D_unconvert(info,&nbuf,buf))
     return(sepwarn(NOT_MET,"trouble converting tag  %s \n",info->name));
free(buf);
}
if(ifrom==impi) free(buf);
#else
return(sepwarn(FAIL_OTHER,"Distribute header requested with a non-mpi version of SEPLIB \n"));
#endif


return(SUCCESS);
}

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_pass_headers(int ithread,char *sep3dname, int ifrom, int ito)
_XFUNCPROTOEND
#endif
{
sep_3d *info;
#ifdef SEP_MPI
MPI_Status status;
#endif
char usage[256],data[256],file[256];
int nsz,i,k;
int n,impi,nbuf;
float o,d;
char temp1[256],temp2[256],temp3[256];
char *buf;


if(ifrom==ito)  return(SUCCESS);
if(ithread!=ifrom && ithread!=ito) return(SUCCESS);
#ifdef SEP_MPI
  MPI_Comm_rank(MPI_COMM_WORLD, &impi);
  info = tag_info_sep3d(sep3dname, INQUIRE);  /* get info on this tag */
if(ifrom==impi){
  if(info == SEPNULL)
    return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",sep3dname));
  
  nbuf=SEP3D_nconvert(info);
    MPI_Send( &nbuf, 1, MPI_INT,ito, 2321, MPI_COMM_WORLD);

}
else MPI_Recv( &nbuf, 1, MPI_INT, ifrom, 2321,MPI_COMM_WORLD,&status);
  buf=(char*) malloc(sizeof(char)*nbuf);
if(ifrom==impi){
    nbuf=0;
   if(0!=SEP3D_convert(info,&nbuf,buf))
     return(sepwarn(NOT_MET,"trouble converting tag  %s \n",info->name));
    MPI_Send( buf, nbuf, MPI_BYTE, ito,2322,MPI_COMM_WORLD);
}
else{
   MPI_Recv( buf, nbuf, MPI_BYTE, ifrom, 2322,MPI_COMM_WORLD,&status);
   nbuf=0;
   if(0!=SEP3D_unconvert(info,&nbuf,buf))
     return(sepwarn(NOT_MET,"trouble unconverting tag %s \n",sep3dname));
}
free(buf);
#else
if(ifrom!=ito)
return(sepwarn(FAIL_OTHER,"Pass header requested with a non-mpi version of SEPLIB \n"));
#endif

return(SUCCESS);
}


#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_broadcast_ints(int *val, int nsz, int ifrom)
_XFUNCPROTOEND
{
int impi;
#endif
#ifdef SEP_MPI
MPI_Status status;
#endif

#ifdef SEP_MPI
 MPI_Bcast(val,nsz,MPI_INT, ifrom, MPI_COMM_WORLD);
#endif
return(SUCCESS);
}




#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_send_int(int ithread,int *val, int ifrom, int ito)
_XFUNCPROTOEND
{
int impi;
#endif
#ifdef SEP_MPI
MPI_Status status;
#endif

if(ifrom==ito)  return(SUCCESS);
if(ithread!=ifrom && ithread!=ito) return(SUCCESS);
#ifdef SEP_MPI
  if(ifrom==ithread) MPI_Send( val, 1, MPI_INT, ito, 2329,MPI_COMM_WORLD);
  else MPI_Recv( val, 1, MPI_INT, ifrom, 2329,MPI_COMM_WORLD,&status);
#else
if(ifrom!=ito)
return(sepwarn(FAIL_OTHER,"Pass integer requested with a non-mpi version of SEPLIB \n"));
#endif
return(SUCCESS);
}






#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_broadcast_data(char *sep3dname,int ifrom,  int esize, int nelem, char *data)
_XFUNCPROTOEND
{
sep_3d *info;
#endif
#ifdef SEP_MPI
MPI_Status status;
#endif
int chunk,done;


done=0;
while(done < nelem*esize){
  chunk=MIN(nelem*esize-done,1000000);
#ifdef SEP_MPI
   MPI_Bcast( (data+done), chunk, MPI_BYTE, ifrom, MPI_COMM_WORLD);
#else
return(sepwarn(FAIL_OTHER,"Broadcast data requested with a non-mpi version of SEPLIB \n"));
#endif
done=done+chunk;
sep_mpi_stop();
}
return(SUCCESS);
}

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_pass_data(int ithread, int ifrom, int ito, int esize, int nelem, char *data)
_XFUNCPROTOEND
{
sep_3d *info;
#endif
#ifdef SEP_MPI
MPI_Status status;
#endif
int done,chunk;

if(ifrom==ito)  return(SUCCESS);
if(ithread!=ifrom && ithread!=ito) return(SUCCESS);
done=0;
while(done < nelem*esize){
  chunk=MIN(nelem*esize-done,1000000);
#ifdef SEP_MPI
  if(ifrom==ithread) MPI_Send( (data+done), chunk, MPI_BYTE, ito, 2329,MPI_COMM_WORLD);
  else MPI_Recv( (data+done), chunk, MPI_BYTE, ifrom, 2329,MPI_COMM_WORLD,&status);
#else
if(ifrom!=ito)
return(sepwarn(FAIL_OTHER,"Pass data requested with a non-mpi version of SEPLIB \n"));
#endif
  done+=chunk;
}

return(SUCCESS);
}

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int SEPDFF_set_mpi_info(sep_dff *info)
_XFUNCPROTOEND
#endif
{
int nlen;
#ifdef SEP_MPI
    MPI_Comm_size(MPI_COMM_WORLD, &info->nmpi);
    MPI_Comm_rank(MPI_COMM_WORLD, &info->impi);
    info->comm=MPI;
#else
info->impi=0;
info->nmpi=1;
info->comm=NFS;
#endif
return(SUCCESS);

}
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
void  sep3d_mpi_init()
_XFUNCPROTOEND
#endif
{
sep_dff *info;
int n,nlen,i;
char temp_ch[1024],temp_send[1024];
char dff_ch[1024],dff_send[1024],junk[1204];


info = tag_info_sepdff("Abc123BCD", INQUIRE);  /* get info on this tag */
if(info==SEPNULL){
  info = tag_info_sepdff("Abc123BCD", SCRATCH);  /* get info on this tag */
  datapath(junk);
  sprintf(dff_ch,"%sDFF_",junk);
  getch("dff","s",dff_ch);
#ifdef SEP_MPI
    MPI_Comm_size(MPI_COMM_WORLD, &info->nmpi);
    MPI_Comm_rank(MPI_COMM_WORLD, &info->impi);
    if(0!=SEPDFF_set_nsect(info,info->nmpi))
      seperr("trouble setting number of sections \n");
    MPI_Get_processor_name (temp_ch,&nlen);
    for(i=0 ; i < info->nmpi; i++){
      if(i==info->impi) strcpy(temp_send,temp_ch);
      nlen=(int)strlen(temp_send);
      MPI_Bcast(&nlen,1,MPI_INT, i, MPI_COMM_WORLD);
      MPI_Bcast(temp_send,nlen,MPI_INT, i, MPI_COMM_WORLD);
      if(0!=SEPDFF_set_mach_sect(info,i,temp_send))
        seperr("trouble szetting mach sect \n");

      if(i==info->impi) strcpy(dff_send,dff_ch);
      nlen=(int)strlen(dff_send);
      MPI_Bcast(&nlen,1,MPI_INT, i, MPI_COMM_WORLD);
      MPI_Bcast(dff_send,nlen,MPI_INT, i, MPI_COMM_WORLD);
      if(0!=SEPDFF_set_tag_sect(info,i,dff_send))
        seperr("trouble szetting tag sect \n");
    }
#else
    if(0!=SEPDFF_set_nsect(info,1))
      seperr("trouble setting number of sections \n");
    if( gethostname(temp_ch,255)!=0 )
       seperr("trouble gettin hostname \n");
    if(0!=SEPDFF_set_mach_sect(info,0,temp_ch))
      seperr("trouble setting mach sect \n");
    if(0!=SEPDFF_set_tag_sect(info,0,dff_ch))
      seperr("trouble setting tag sect \n");
#endif
}
}
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_make_local_name(char *tagin, char *tagout)
_XFUNCPROTOEND
#endif
{
sep_dff *junk;
junk = tag_info_sepdff("Abc123BCD", INQUIRE);  /* get info on this tag */
if(junk==SEPNULL)
  return(sepwarn(NOT_MET,"ERROR: INIT NOT CALLED \n"));
  sprintf(tagout,"%s%s_XXXXXX",junk->tag_sect[sep_thread_num()],tagin);
  mkstemp(tagout);
return(0);
}

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int SEPDFF_grab_mpi_names(sep_dff *info, char **machnames, char **dff_path)
_XFUNCPROTOEND
#endif
{
int inlen;
int nlen,i;
sep_dff *junk;

junk = tag_info_sepdff("Abc123BCD", INQUIRE);  /* get info on this tag */
if(junk==SEPNULL)
  return(sepwarn(NOT_MET,"ERROR: INIT NOT CALLED \n"));
for(i=0; i < info->nmpi; i++){
  strcpy(machnames[i],junk->mach_sect[i]);
  strcpy(dff_path[i],junk->tag_sect[i]);
}
return(SUCCESS);
}

int SEPDFF_condense_buffer(sep_dff *info, int ito, char *op, float *buf_in,float *buf_out, int n1, int n2){
int done,chunk;
int i;
int oper;

done=0;

#ifdef SEP_MPI
if(op[0]=='+') oper=MPI_SUM;
else return(sepwarn(NOT_MET,"Don't understand condense operation %s \n",op));
#endif


while(done < n1*n2/4){
  chunk=MIN(n1*n2/4-done,1000000);
#ifdef SEP_MPI
   MPI_Reduce((buf_in+done),(buf_out+done),chunk,MPI_FLOAT,MPI_SUM,ito,
    MPI_COMM_WORLD);
#endif
 sep_mpi_stop();
 done+=chunk;
}

return(0);
}
int sep3d_broadcast_string(char *string, int ifrom){
int nlen;

nlen=(int)(strlen(string)+1);
#ifdef SEP_MPI
 MPI_Barrier(MPI_COMM_WORLD);
 MPI_Bcast(&nlen,1,MPI_INT, ifrom, MPI_COMM_WORLD);
 MPI_Barrier(MPI_COMM_WORLD);
 MPI_Bcast(string,nlen,MPI_INT, ifrom, MPI_COMM_WORLD);
 MPI_Barrier(MPI_COMM_WORLD);
#endif


return(SUCCESS);
}
void sep_mpi_stop(){

#ifdef SEP_MPI
MPI_Barrier(MPI_COMM_WORLD);
#endif


}
int sep_num_thread(){
int i;
#ifdef SEP_MPI
MPI_Comm_size(MPI_COMM_WORLD, &i);
return(i);
#else
return(1);
#endif
}



int sep_thread_num(){
int i;
#ifdef SEP_MPI
    MPI_Comm_rank(MPI_COMM_WORLD, &i);
   return(i);
#else
return(0);
#endif
}

==> seplib-6.3.8.tar.gz: seplib-6.3.8/seplib_base/lib/corelibs/superset/sect_transfer_mpi.c <==
#define SET_SDOC 1
#include<sitedef.h>
#include<superset.h>
#include<superset_internal.h>
#ifdef SEP_MPI
#include<mpi.h>
#endif
#ifndef MAX
#define MAX(a,b) ( ((a)>(b)) ? (a):(b) )
#endif
#ifndef MIN
#define MIN(a,b) ( ((a)<(b)) ? (a):(b) )
#endif



#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_broadcast_headers(char *sep3dname, int ifrom)
_XFUNCPROTOEND
#endif
{
sep_3d *info;
#ifdef SEP_MPI
MPI_Status status;
MPI_Comm comm;
#endif
char usage[256],data[256],file[256];
int nsz,i,k,iam,impi;
int n,nbuf;
float o,d;
char temp1[256],temp2[256],temp3[256];
char *buf;


#ifdef SEP_MPI
  MPI_Comm_rank(MPI_COMM_WORLD, &impi);
if(ifrom==impi){
  info = tag_info_sep3d(sep3dname, INQUIRE);  /* get info on this tag */
  if(info == SEPNULL)
    return (sepwarn(INVALID_STRUC,"tag:%s 2invalid struc\n",sep3dname));
  
  nbuf=SEP3D_nconvert(info);

}
else{
  info = tag_info_sep3d(sep3dname, INQUIRE);  /* get info on this tag */
  if(info == SEPNULL){
    info = tag_info_sep3d(sep3dname, SCRATCH);  /* get info on this tag */
     if(info == SEPNULL)
        return (sepwarn(INVALID_STRUC,"tag:%s 1invalid struc\n",sep3dname));
   }
  

}
  MPI_Bcast( &nbuf, 1, MPI_INT,  ifrom,MPI_COMM_WORLD); 
  sep_mpi_stop();
  buf=(char*) malloc(sizeof(char)*(nbuf));
if(ifrom==impi){
    nbuf=0;
   if(0!=SEP3D_convert(info,&nbuf,buf))
     return(sepwarn(NOT_MET,"trouble converting tag  %s \n",info->name));
}
  MPI_Bcast((void*)buf,nbuf,MPI_BYTE, ifrom, MPI_COMM_WORLD);
sep_mpi_stop();
if(ifrom!=impi){
    nbuf=0;
   if(0!=SEP3D_unconvert(info,&nbuf,buf))
     return(sepwarn(NOT_MET,"trouble converting tag  %s \n",info->name));
free(buf);
}
if(ifrom==impi) free(buf);
#else
return(sepwarn(FAIL_OTHER,"Distribute header requested with a non-mpi version of SEPLIB \n"));
#endif


return(SUCCESS);
}

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_pass_headers(int ithread,char *sep3dname, int ifrom, int ito)
_XFUNCPROTOEND
#endif
{
sep_3d *info;
#ifdef SEP_MPI
MPI_Status status;
#endif
char usage[256],data[256],file[256];
int nsz,i,k;
int n,impi,nbuf;
float o,d;
char temp1[256],temp2[256],temp3[256];
char *buf;


if(ifrom==ito)  return(SUCCESS);
if(ithread!=ifrom && ithread!=ito) return(SUCCESS);
#ifdef SEP_MPI
  MPI_Comm_rank(MPI_COMM_WORLD, &impi);
  info = tag_info_sep3d(sep3dname, INQUIRE);  /* get info on this tag */
if(ifrom==impi){
  if(info == SEPNULL)
    return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",sep3dname));
  
  nbuf=SEP3D_nconvert(info);
    MPI_Send( &nbuf, 1, MPI_INT,ito, 2321, MPI_COMM_WORLD);

}
else MPI_Recv( &nbuf, 1, MPI_INT, ifrom, 2321,MPI_COMM_WORLD,&status);
  buf=(char*) malloc(sizeof(char)*nbuf);
if(ifrom==impi){
    nbuf=0;
   if(0!=SEP3D_convert(info,&nbuf,buf))
     return(sepwarn(NOT_MET,"trouble converting tag  %s \n",info->name));
    MPI_Send( buf, nbuf, MPI_BYTE, ito,2322,MPI_COMM_WORLD);
}
else{
   MPI_Recv( buf, nbuf, MPI_BYTE, ifrom, 2322,MPI_COMM_WORLD,&status);
   nbuf=0;
   if(0!=SEP3D_unconvert(info,&nbuf,buf))
     return(sepwarn(NOT_MET,"trouble unconverting tag %s \n",sep3dname));
}
free(buf);
#else
if(ifrom!=ito)
return(sepwarn(FAIL_OTHER,"Pass header requested with a non-mpi version of SEPLIB \n"));
#endif

return(SUCCESS);
}


#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_broadcast_ints(int *val, int nsz, int ifrom)
_XFUNCPROTOEND
{
int impi;
#endif
#ifdef SEP_MPI
MPI_Status status;
#endif

#ifdef SEP_MPI
 MPI_Bcast(val,nsz,MPI_INT, ifrom, MPI_COMM_WORLD);
#endif
return(SUCCESS);
}




#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_send_int(int ithread,int *val, int ifrom, int ito)
_XFUNCPROTOEND
{
int impi;
#endif
#ifdef SEP_MPI
MPI_Status status;
#endif

if(ifrom==ito)  return(SUCCESS);
if(ithread!=ifrom && ithread!=ito) return(SUCCESS);
#ifdef SEP_MPI
  if(ifrom==ithread) MPI_Send( val, 1, MPI_INT, ito, 2329,MPI_COMM_WORLD);
  else MPI_Recv( val, 1, MPI_INT, ifrom, 2329,MPI_COMM_WORLD,&status);
#else
if(ifrom!=ito)
return(sepwarn(FAIL_OTHER,"Pass integer requested with a non-mpi version of SEPLIB \n"));
#endif
return(SUCCESS);
}






#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_broadcast_data(char *sep3dname,int ifrom,  int esize, int nelem, char *data)
_XFUNCPROTOEND
{
sep_3d *info;
#endif
#ifdef SEP_MPI
MPI_Status status;
#endif
int chunk,done;


done=0;
while(done < nelem*esize){
  chunk=MIN(nelem*esize-done,1000000);
#ifdef SEP_MPI
   MPI_Bcast( (data+done), chunk, MPI_BYTE, ifrom, MPI_COMM_WORLD);
#else
return(sepwarn(FAIL_OTHER,"Broadcast data requested with a non-mpi version of SEPLIB \n"));
#endif
done=done+chunk;
sep_mpi_stop();
}
return(SUCCESS);
}

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_pass_data(int ithread, int ifrom, int ito, int esize, int nelem, char *data)
_XFUNCPROTOEND
{
sep_3d *info;
#endif
#ifdef SEP_MPI
MPI_Status status;
#endif
int done,chunk;

if(ifrom==ito)  return(SUCCESS);
if(ithread!=ifrom && ithread!=ito) return(SUCCESS);
done=0;
while(done < nelem*esize){
  chunk=MIN(nelem*esize-done,1000000);
#ifdef SEP_MPI
  if(ifrom==ithread) MPI_Send( (data+done), chunk, MPI_BYTE, ito, 2329,MPI_COMM_WORLD);
  else MPI_Recv( (data+done), chunk, MPI_BYTE, ifrom, 2329,MPI_COMM_WORLD,&status);
#else
if(ifrom!=ito)
return(sepwarn(FAIL_OTHER,"Pass data requested with a non-mpi version of SEPLIB \n"));
#endif
  done+=chunk;
}

return(SUCCESS);
}

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int SEPDFF_set_mpi_info(sep_dff *info)
_XFUNCPROTOEND
#endif
{
int nlen;
#ifdef SEP_MPI
    MPI_Comm_size(MPI_COMM_WORLD, &info->nmpi);
    MPI_Comm_rank(MPI_COMM_WORLD, &info->impi);
    info->comm=MPI;
#else
info->impi=0;
info->nmpi=1;
info->comm=NFS;
#endif
return(SUCCESS);

}
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
void  sep3d_mpi_init()
_XFUNCPROTOEND
#endif
{
sep_dff *info;
int n,nlen,i;
char temp_ch[1024],temp_send[1024];
char dff_ch[1024],dff_send[1024],junk[1204];


info = tag_info_sepdff("Abc123BCD", INQUIRE);  /* get info on this tag */
if(info==SEPNULL){
  info = tag_info_sepdff("Abc123BCD", SCRATCH);  /* get info on this tag */
  datapath(junk);
  sprintf(dff_ch,"%sRaNd_",junk);
  getch("dff","s",dff_ch);
#ifdef SEP_MPI
    MPI_Comm_size(MPI_COMM_WORLD, &info->nmpi);
    MPI_Comm_rank(MPI_COMM_WORLD, &info->impi);
    if(0!=SEPDFF_set_nsect(info,info->nmpi))
      seperr("trouble setting number of sections \n");
    MPI_Get_processor_name (temp_ch,&nlen);
    for(i=0 ; i < info->nmpi; i++){
      if(i==info->impi) strcpy(temp_send,temp_ch);
      nlen=(int)strlen(temp_send);
      MPI_Bcast(&nlen,1,MPI_INT, i, MPI_COMM_WORLD);
      MPI_Bcast(temp_send,nlen,MPI_INT, i, MPI_COMM_WORLD);
      if(0!=SEPDFF_set_mach_sect(info,i,temp_send))
        seperr("trouble szetting mach sect \n");

      if(i==info->impi) strcpy(dff_send,dff_ch);
      nlen=(int)strlen(dff_send);
      MPI_Bcast(&nlen,1,MPI_INT, i, MPI_COMM_WORLD);
      MPI_Bcast(dff_send,nlen,MPI_INT, i, MPI_COMM_WORLD);
      if(0!=SEPDFF_set_tag_sect(info,i,dff_send))
        seperr("trouble szetting tag sect \n");
    }
#else
    if(0!=SEPDFF_set_nsect(info,1))
      seperr("trouble setting number of sections \n");
    if( gethostname(temp_ch,255)!=0 )
       seperr("trouble gettin hostname \n");
    if(0!=SEPDFF_set_mach_sect(info,0,temp_ch))
      seperr("trouble setting mach sect \n");
    if(0!=SEPDFF_set_tag_sect(info,0,dff_ch))
      seperr("trouble setting tag sect \n");
#endif
}
}
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_make_local_name(char *tagin, char *tagout)
_XFUNCPROTOEND
#endif
{
sep_dff *junk;
junk = tag_info_sepdff("Abc123BCD", INQUIRE);  /* get info on this tag */
if(junk==SEPNULL)
  return(sepwarn(NOT_MET,"ERROR: INIT NOT CALLED \n"));
  sprintf(tagout,"%s%s_XXXXXX",junk->tag_sect[sep_thread_num()],tagin);
  mkstemp(tagout);
return(0);
}

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int SEPDFF_grab_mpi_names(sep_dff *info, char **machnames, char **dff_path)
_XFUNCPROTOEND
#endif
{
int inlen;
int nlen,i;
sep_dff *junk;

junk = tag_info_sepdff("Abc123BCD", INQUIRE);  /* get info on this tag */
if(junk==SEPNULL)
  return(sepwarn(NOT_MET,"ERROR: INIT NOT CALLED \n"));
for(i=0; i < info->nmpi; i++){
  strcpy(machnames[i],junk->mach_sect[i]);
  strcpy(dff_path[i],junk->tag_sect[i]);
}
return(SUCCESS);
}

int SEPDFF_condense_buffer(sep_dff *info, int ito, char *op, float *buf_in,float *buf_out, int n1, int n2){
int done,chunk;
int i;
#ifdef SEP_MPI
MPI_Op oper;
#endif

done=0;

#ifdef SEP_MPI
if(op[0]=='+') oper=MPI_SUM;
else return(sepwarn(NOT_MET,"Don't understand condense operation %s \n",op));
#endif


while(done < n1*n2/4){
  chunk=MIN(n1*n2/4-done,1000000);
#ifdef SEP_MPI
   MPI_Reduce((buf_in+done),(buf_out+done),chunk,MPI_FLOAT,MPI_SUM,ito,
    MPI_COMM_WORLD);
#endif
 sep_mpi_stop();
 done+=chunk;
}

return(0);
}
int sep3d_broadcast_string(char *string, int ifrom){
int nlen;

nlen=(int)(strlen(string)+1);
#ifdef SEP_MPI
 MPI_Barrier(MPI_COMM_WORLD);
 MPI_Bcast(&nlen,1,MPI_INT, ifrom, MPI_COMM_WORLD);
 MPI_Barrier(MPI_COMM_WORLD);
 MPI_Bcast(string,nlen,MPI_INT, ifrom, MPI_COMM_WORLD);
 MPI_Barrier(MPI_COMM_WORLD);
#endif


return(SUCCESS);
}
void sep_mpi_stop(){

#ifdef SEP_MPI
MPI_Barrier(MPI_COMM_WORLD);
#endif


}
int sep_num_thread(){
int i;
#ifdef SEP_MPI
MPI_Comm_size(MPI_COMM_WORLD, &i);
return(i);
#else
return(1);
#endif
}



int sep_thread_num(){
int i;
#ifdef SEP_MPI
    MPI_Comm_rank(MPI_COMM_WORLD, &i);
   return(i);
#else
return(0);
#endif
}
